#include "mbed.h"

#ifndef __SOUND_H__
#define __SOUND_H__


// Notes in Hertz
enum NOTE
{
    NOTE_C3 = 130,
    NOTE_Cs3 = 138,
    NOTE_Db3 = NOTE_Cs3,
    NOTE_D3 = 146,
    NOTE_Ds3 = 155,
    NOTE_Eb3 = NOTE_Ds3,
    NOTE_E3 = 164,
    NOTE_F3 = 174,
    NOTE_Fs3 = 184,
    NOTE_Gb3 = NOTE_Fs3,
    NOTE_G3 = 195,
    NOTE_Gs3 = 207,
    NOTE_Ab3 = NOTE_Gs3,
    NOTE_A3 = 220,
    NOTE_As3 = 233,
    NOTE_Bb3 = NOTE_As3,
    NOTE_B3 = 246,
    NOTE_C4 = 262,
    NOTE_Cs4 = 277,
    NOTE_Db4 = NOTE_Cs4,
    NOTE_D4 = 294,
    NOTE_Ds4 = 311,
    NOTE_Eb4 = NOTE_Ds4,
    NOTE_E4 = 330,
    NOTE_F4 = 349,
    NOTE_Fs4 = 370,
    NOTE_Gb4 = NOTE_Fs4,
    NOTE_G4 = 392,
    NOTE_Gs4 = 415,
    NOTE_Ab4 = NOTE_Gs4,
    NOTE_A4 = 440,
    NOTE_As4 = 466,
    NOTE_B4 = 494,
    NOTE_C5 = 523,
    NOTE_Cs5 = 554,
    NOTE_Db5 = NOTE_Cs5,
    NOTE_D5 = 587,
    NOTE_Ds5 = 622,
    NOTE_Eb5 = NOTE_Ds5,
    NOTE_E5 = 659,
    NOTE_F5 = 698,
    NOTE_Fs5 = 740,
    NOTE_Gb5 = NOTE_Fs5,
    NOTE_G5 = 784,
    NOTE_Gs5 = 831,
    NOTE_Ab5 = NOTE_Gs5,
    NOTE_A5 = 880,
    NOTE_As5 = 932,
    NOTE_Bb5 = NOTE_As5,
    NOTE_B5 = 988,
    NOTE_C6 = 1047,
    NOTE_Cs6 = 1109,
    NOTE_Db6 = NOTE_Cs6,
    NOTE_D6 = 1175,
    NOTE_Ds6 = 1245,
    NOTE_Eb6 = NOTE_Ds6,
    NOTE_E6 = 1319,
    NOTE_F6 = 1397,
    NOTE_Fs6 = 1480,
    NOTE_Gb6 = NOTE_Fs6,
    NOTE_G6 = 1568,
    NOTE_Gs6 = 1661,
    NOTE_Ab6 = NOTE_Gs6,
    NOTE_A6 = 1760,
    NOTE_As6 = 1865,
    NOTE_Bb6 = NOTE_As6,
    NOTE_B6 = 1976,
    NOTE_C7 = 2093,
    NOTE_Cs7 = 2217,
    NOTE_Db7 = NOTE_Cs7,
    NOTE_D7 = 2349,
    NOTE_Ds7 = 2489,
    NOTE_F7 = 2793,
    NOTE_Fs7 = 2959,
    NOTE_Gb7 = NOTE_Fs7,
    NOTE_G7 = 3135,
    NOTE_Gs7 = 3322,
    NOTE_Ab7 = NOTE_Gs7,
    NOTE_A7 = 3520,
    NOTE_As7 = 3729,
    NOTE_Bb7 = NOTE_As7,
    NOTE_B7 = 3951,
    NOTE_C8 = 4186
};

#define SOUND_MAX_CHANNELS  4

enum SOUND_INST_WAVEFORM
{
    SOUND_SQUARE,
    SOUND_SAW,
    SOUND_TRIANGLE,
    SOUND_SINE,
    SOUND_NOISE,
};

struct SOUND_CHANNEL
{
    bool active;
    
    unsigned short orig_phase_inc;
    unsigned short phase_inc;
    unsigned short phase_acc;
    unsigned char volume;
    
    unsigned char play_length;             // 0 to 255 in 50Hz ticks
    
    // Instrument parameters
    unsigned char  vol_level;              // Starting volume 0 to 64
    char            waveform;              // 0 = square, 1 = saw, 2 = triangle, 3 = sine, 4 = noise
    unsigned char   decay_amount;          // 0 to 255
    signed char     slide;                 // +- 127
    unsigned short  change_amount;         // +-127
    char            change_speed;          // +-127
    unsigned char   repeat_speed;          // 0 to 255
    unsigned char   vibrato_depth;         // Amplitude of Vibrato => 0 to 255
    unsigned char   vibrato_speed;         // Frequency of the Vibrato
    
    // Extra state stuff that makes the instrument do its thing
    bool use_noise;
    unsigned char *pWaveform;
    
    unsigned char vibrato_phase;
    short int vibrato_inc;
    unsigned short repeat_wait;
    unsigned short change_wait;
    unsigned short change_inc;
};

class Sound
{
public:

    Sound();
    ~Sound();
    
    void Init();
    
    void PlaySong(char *pRTTTL, int iChannel = 0, int iBPM = 120, bool bLoop = true);
    void SetSongTempo(int iBPM);
    void TransposeSong(int iOffset);
    void StopSong(void);
    bool SongIsPlaying(void) { return bPlaySong; };
    
    // TODO - create a set instrument that allows the parameters to be encoded
    // TODO - what else??? we have sound effects, we have songs... drums?
    void SetInstrument(int iChannel, int iWaveform, int iVolume, int iDecayTime, int iSlide, int iChangeAmount, int iChangeSpeed, int iRepeatSpeed, int iVibratoDepth, int iVibratoSpeed);
    void PlaySound(int iChannel, int iFreqHz, int iLength = 0);
    void StopSound(int iChannel);
    bool IsBusy(int iChannel);
    
    void Mute(bool mute);
    bool IsMuted(void) { return bMuted; };

    int GetCounter(void);
    
    friend void TIMER32_0_IRQHandler(void);
    
protected:

    void SetupDefaults(void);
    void DoEffects(void);
    
    SOUND_CHANNEL channel[SOUND_MAX_CHANNELS];
    
    unsigned short effect_wait;
    
    // Sound Related variables
    bool bMuted;
    
    // Song Related Variables and Functions
    bool bPlaySong;
    bool bLoopSong;
    int iSongChannel;
    char *m_pSong;
    char *m_pSongBuffer;
    int iSongBufferLen;
    int iSongNoteOffset;
    int iNoteCounter;
    int iTicksPerBeat;
    void DoSong(void);
    int SongGetFreq(char note, int octave, bool bSharp);
};

#endif // __SOUND_H__
